/**
 * class for psvm
 */
package psvm;

import java.util.Vector;
import java.util.Iterator;
import java.util.Collections;
import java.util.StringTokenizer;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.lang.Math;

import libsvm.svm;
import libsvm.svm_model;
import libsvm.svm_node;
import libsvm.svm_parameter;
import libsvm.svm_print_interface;
import libsvm.svm_problem;


/**
 * This is the basic class for psvm
 */
public class DataSet {
	svm psvm_ = null;
	/**
	 * construct an empty svm class
	 */
	public DataSet() { // create an empty svm object
		psvm_ = new svm();
	}


    /**
     * save the weighted samples for old version libsvm 2.8.1
     */
    public void saveProblem(svm_problem prob, String sample) {
        try {
            FileWriter out1 = new FileWriter(sample);   // writer for samples file

            for(int i = 0; i < prob.l; i ++) {
                int label = (int)prob.y[i];

             	out1.write(String.valueOf(prob.W[i]));
             	out1.write(" ");
                out1.write(String.valueOf(label));    // output class label

                for(int j = 0; j < prob.x[i].length; j++) { // reconstruct samples features vectors file
					// output features vectors for each sample
                    out1.write(" ");
                    out1.write(String.valueOf(prob.x[i][j].index));
                    out1.write(":");
                    out1.write(String.valueOf(prob.x[i][j].value));
                }

                out1.write("\n");
            }

            out1.close();

        } catch (IOException ex) {
            System.out.println(ex.getMessage());
            System.exit(1);
        }
    }  


    /**
     * save the weighted samples and weight to files
     */
    public void saveProblem(svm_problem prob, String sample, String weight) {
        try {
            FileWriter out1 = new FileWriter(sample);   // writer for samples file
            FileWriter out2 = new FileWriter(weight);   // writer for weights file 

            for(int i = 0; i < prob.l; i ++) {
                int label = (int)prob.y[i];

                out1.write(String.valueOf(label));    // output class label

                for(int j = 0; j < prob.x[i].length; j++) { // reconstruct samples features vectors file
					// output features vectors for each sample
                    out1.write(" ");
                    out1.write(String.valueOf(prob.x[i][j].index));
                    out1.write(":");
                    out1.write(String.valueOf(prob.x[i][j].value));
                }

                out1.write("\n");

				// output weight for each sample
             	out2.write(String.valueOf(prob.W[i]));
              	out2.write("\n");
            }

            out1.close();
            out2.close();

        } catch (IOException ex) {
            System.out.println(ex.getMessage());
            System.exit(1);
        }
    }  


	/**
	 *  read samples from file and return a svm problem to represent the samples
	 *  this part is copied from libsvm
	 */
	public svm_problem readSamples(String fileName) {
		svm_problem prob = new svm_problem();

		try {
			BufferedReader in = new BufferedReader(new FileReader(fileName));
			Vector<Double> vy = new Vector<Double>();
			Vector<svm_node[]> vx = new Vector<svm_node[]>();
			int max_index = 0;

			while(true) {
				String line = in.readLine();
				System.out.println(line);
				if(line == null) break;

				int cut = line.indexOf(" #");   // ignore the comment
				if(cut > -1) {
					line = line.substring(0, cut);
				}

				StringTokenizer st = new StringTokenizer(line," \t\n\r\f:"); // split line by dilim

				vy.addElement(Double.parseDouble(st.nextToken())); // read label

				int m = st.countTokens() / 2;
				svm_node[] x = new svm_node[m];
				for(int j = 0; j < m; j++) {
					x[j] = new svm_node();
					x[j].index = Integer.valueOf(st.nextToken());
					x[j].value = Double.parseDouble(st.nextToken());
				}

				if(m > 0) max_index = Math.max(max_index, x[m-1].index); // get the max column number
				vx.addElement(x);
			}

			in.close();

			prob.l = vy.size();
			prob.x = new svm_node[prob.l][];

			for(int i = 0; i < prob.l; i++) {
				prob.x[i] = vx.elementAt(i);
			}

			prob.y = new double[prob.l];
			prob.W = new double[prob.l];

			for(int i = 0; i < prob.l; i++) {
				prob.y[i] = vy.elementAt(i);
				prob.W[i] = 1.0;  // set default weight for each sample to 1.0
			}

		} catch (IOException ex) {
            System.out.println(ex.getMessage());
            System.exit(1);
        }

		return prob;
	}
}

